@@ -1,6 +1,6 @@
 # Author:: Nathaniel Talbott.
 # Copyright:: Copyright (c) 2000-2003 Nathaniel Talbott. All rights reserved.
-#             Copyright (c) 2009-2010 Kouhei Sutou. All rights reserved.
+#             Copyright (c) 2009-2012 Kouhei Sutou. All rights reserved.
 # License:: Ruby license.
 
 require 'test/unit/assertionfailederror'
@@ -62,24 +62,72 @@ module Test
       public
       def assert(boolean, message=nil)
         _wrap_assertion do
+          assertion_message = nil
           case message
           when nil, String, Proc
+          when AssertionMessage
+            assertion_message = message
           else
-            error_message = "assertion message must be String or Proc: "
+            error_message = "assertion message must be String, Proc or "
+            error_message << "#{AssertionMessage}: "
             error_message << "<#{message.inspect}>(<#{message.class}>)"
             raise ArgumentError, error_message, filter_backtrace(caller)
           end
           assert_block("assert should not be called with a block.") do
             !block_given?
           end
-          assert_block(build_message(message, "<?> is not true.", boolean)) do
+          assertion_message ||= build_message(message,
+                                              "<?> is not true.",
+                                              boolean)
+          assert_block(assertion_message) do
             boolean
           end
         end
       end
 
+      # Asserts that +object+ is false or nil.
+      #
+      # @note Just for minitest compatibility. :<
+      #
+      # @param [Object] object The object to be asserted.
+      # @return [void]
+      #
+      # @example Pass patterns
+      #   refute(false)    # => pass
+      #   refute(nil)      # => pass
+      #
+      # @example Fialure patterns
+      #   refute(true)     # => failure
+      #   refute("string") # => failure
+      #
+      # @since 2.5.3
+      def refute(object, message=nil)
+        _wrap_assertion do
+          assertion_message = nil
+          case message
+          when nil, String, Proc
+          when AssertionMessage
+            assertion_message = message
+          else
+            error_message = "assertion message must be String, Proc or "
+            error_message << "#{AssertionMessage}: "
+            error_message << "<#{message.inspect}>(<#{message.class}>)"
+            raise ArgumentError, error_message, filter_backtrace(caller)
+          end
+          assert_block("refute should not be called with a block.") do
+            !block_given?
+          end
+          assertion_message ||= build_message(message,
+                                              "<?> is neither nil or false.",
+                                              object)
+          assert_block(assertion_message) do
+            not object
+          end
+        end
+      end
+
       ##
-      # Passes if +expected+ == +actual.
+      # Passes if +expected+ == +actual+.
       #
       # Note that the ordering of arguments is important, since a helpful
       # error message is generated when this one fails that tells you the
@@ -111,11 +159,7 @@ EOT
         begin
           assert_block(full_message) { expected == actual }
         rescue AssertionFailedError => failure
-          failure.expected = expected
-          failure.actual = actual
-          failure.inspected_expected = AssertionMessage.convert(expected)
-          failure.inspected_actual = AssertionMessage.convert(actual)
-          failure.user_message = message
+          _set_failed_information(failure, expected, actual, message)
           raise failure # For JRuby. :<
         end
       end
@@ -142,25 +186,26 @@ EOT
         assert_expected_exception = Proc.new do |*_args|
           message, assert_exception_helper, actual_exception = _args
           expected = assert_exception_helper.expected_exceptions
+          diff = AssertionMessage.delayed_diff(expected, actual_exception)
           full_message = build_message(message,
-                                       "<?> exception expected but was\n?",
-                                       expected, actual_exception)
-          assert_block(full_message) do
-            expected == [] or assert_exception_helper.expected?(actual_exception)
+                                       "<?> exception expected but was\n<?>.?",
+                                       expected, actual_exception, diff)
+          begin
+            assert_block(full_message) do
+              expected == [] or
+                assert_exception_helper.expected?(actual_exception)
+            end
+          rescue AssertionFailedError => failure
+            _set_failed_information(failure, expected, actual_exception,
+                                    message)
+            raise failure # For JRuby. :<
           end
         end
         _assert_raise(assert_expected_exception, *args, &block)
       end
 
-      ##
-      # Alias of assert_raise.
-      #
-      # Will be deprecated in 1.9, and removed in 2.0.
-
-      public
-      def assert_raises(*args, &block)
-        assert_raise(*args, &block)
-      end
+      # Just for minitest compatibility. :<
+      alias_method :assert_raises, :assert_raise
 
       ##
       # Passes if the block raises one of the given
@@ -176,7 +221,7 @@ EOT
           expected = assert_exception_helper.expected_exceptions
           full_message = build_message(message,
                                        "<?> family exception expected " +
-                                       "but was\n?",
+                                       "but was\n<?>.",
                                        expected, actual_exception)
           assert_block(full_message) do
             assert_exception_helper.expected?(actual_exception, :kind_of?)
@@ -333,6 +378,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_respond_to, :assert_not_respond_to
+
       ##
       # Passes if +string+ =~ +pattern+.
       #
@@ -455,6 +505,11 @@ EOT
         assert_block(full_message) { !actual.equal?(expected) }
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_same, :assert_not_same
+
       ##
       # Passes if +expected+ != +actual+
       #
@@ -467,6 +522,11 @@ EOT
         assert_block(full_message) { expected != actual }
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_equal, :assert_not_equal
+
       ##
       # Passes if ! +object+ .nil?
       #
@@ -479,6 +539,11 @@ EOT
         assert_block(full_message){!object.nil?}
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_nil, :assert_not_nil
+
       ##
       # Passes if +regexp+ !~ +string+
       #
@@ -499,6 +564,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_match, :assert_not_match
+
       ##
       # Deprecated. Use #assert_not_match instead.
       #
@@ -567,13 +637,10 @@ EOT
         end
       end
 
-      ##
-      # Alias of assert_throw.
+      # Just for minitest compatibility. :<
       #
-      # Will be deprecated in 1.9, and removed in 2.0.
-      def assert_throws(*args, &block)
-        assert_throw(*args, &block)
-      end
+      # @since 2.5.3
+      alias_method :assert_throws, :assert_throw
 
       ##
       # Passes if block does not throw anything.
@@ -650,6 +717,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_in_delta, :assert_not_in_delta
+
       # :stopdoc:
       private
       def _assert_in_delta_validate_arguments(expected_float,
@@ -746,7 +818,11 @@ EOT
                                                     message)
           assert_block(full_message) do
             normalized_expected_float = expected_float.to_f
-            delta = normalized_expected_float * epsilon.to_f
+            if normalized_expected_float.zero?
+              delta = epsilon.to_f ** 2
+            else
+              delta = normalized_expected_float * epsilon.to_f
+            end
             (normalized_expected_float - actual_float.to_f).abs <= delta
           end
         end
@@ -1270,6 +1346,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :assert_includes, :assert_include
+
       ##
       # Passes if +collection+ doesn't include +object+.
       #
@@ -1364,11 +1445,14 @@ EOT
         end
       end
 
-      ##
-      # Called whenever an assertion is made.  Define this in classes that
-      # include Test::Unit::Assertions to record assertion counts.
-
-      private
+      public
+      # Called whenever an assertion is made. Define this in classes
+      # that include Test::Unit::Assertions to record assertion
+      # counts.
+      #
+      # This is a public API for developers who extend test-unit.
+      #
+      # @return [void]
       def add_assertion
       end
 
@@ -1412,6 +1496,15 @@ EOT
         end
       end
 
+      private
+      def _set_failed_information(failure, expected, actual, user_message)
+        failure.expected = expected
+        failure.actual = actual
+        failure.inspected_expected = AssertionMessage.convert(expected)
+        failure.inspected_actual = AssertionMessage.convert(actual)
+        failure.user_message = user_message
+      end
+
       class AssertionMessage
         @use_pp = true
         class << self
@@ -1504,30 +1597,22 @@ EOT
           end
 
           def convert(object)
-            case object
-            when Exception
-              <<EOM.chop
-Class: <#{convert(object.class)}>
-Message: <#{convert(object.message)}>
----Backtrace---
-#{Util::BacktraceFilter.filter_backtrace(object.backtrace).join("\n")}
----------------
-EOM
-            else
-              inspector = Inspector.new(object)
-              if use_pp
-                begin
-                  require 'pp' unless defined?(PP)
-                  begin
-                    return PP.pp(inspector, '').chomp
-                  rescue NameError
-                  end
-                rescue LoadError
-                  self.use_pp = false
-                end
-              end
-              inspector.inspect
+            if object.is_a?(Exception)
+              object = AssertExceptionHelper::WrappedException.new(object)
             end
+            inspector = Inspector.new(object)
+            if use_pp
+              begin
+                require 'pp' unless defined?(PP)
+                begin
+                  return PP.pp(inspector, '').chomp
+                rescue NameError
+                end
+              rescue LoadError
+                self.use_pp = false
+              end
+            end
+            inspector.inspect
           end
         end
 
@@ -1539,6 +1624,19 @@ EOM
               inspected_objects[object.object_id] ||=
                 new(object, inspected_objects)
             end
+
+            @@inspector_classes = []
+            def inspector_classes
+              @@inspector_classes
+            end
+
+            def register_inspector_class(inspector_class)
+              @@inspector_classes << inspector_class
+            end
+
+            def unregister_inspector_class(inspector_class)
+              @@inspector_classes.delete(inspector_class)
+            end
           end
 
           attr_reader :object
@@ -1572,20 +1670,21 @@ EOM
 
           private
           def inspect_target
-            if HashInspector.target?(@object)
-              HashInspector.new(@object, @inspected_objects)
-            elsif ArrayInspector.target?(@object)
-              ArrayInspector.new(@object, @inspected_objects)
-            else
-              @object
+            self.class.inspector_classes.each do |inspector_class|
+              if inspector_class.target?(@object)
+                return inspector_class.new(@object, @inspected_objects)
+              end
             end
+            @object
           end
         end
 
         class HashInspector
+          Inspector.register_inspector_class(self)
+
           class << self
             def target?(object)
-              object.is_a?(Hash) or object == ENV
+              object.is_a?(Hash) or ENV.equal?(object)
             end
           end
 
@@ -1635,6 +1734,8 @@ EOM
         end
 
         class ArrayInspector
+          Inspector.register_inspector_class(self)
+
           class << self
             def target?(object)
               object.is_a?(Array)
@@ -1723,7 +1824,30 @@ EOM
           def result(parameters)
             raise "The number of parameters does not match the number of substitutions." if(parameters.size != count)
             params = parameters.dup
-            @parts.collect{|e| e == '?' ? params.shift : e.gsub(/\\\?/m, '?')}.join('')
+            expanded_template = ""
+            @parts.each do |part|
+              if part == '?'
+                encoding_safe_concat(expanded_template, params.shift)
+              else
+                expanded_template << part.gsub(/\\\?/m, '?')
+              end
+            end
+            expanded_template
+          end
+
+          private
+          if Object.const_defined?(:Encoding)
+            def encoding_safe_concat(buffer, parameter)
+              if Encoding.compatible?(buffer, parameter)
+                buffer << parameter
+              else
+                buffer << parameter.dup.force_encoding(buffer.encoding)
+              end
+            end
+          else
+            def encoding_safe_concat(buffer, parameter)
+              buffer << parameter
+            end
           end
         end
 
@@ -1750,7 +1874,7 @@ EOM
         def to_s
           message_parts = []
           if (@head)
-            head = @head.to_s 
+            head = @head.to_s
             unless(head.empty?)
               message_parts << add_period(head)
             end
@@ -1763,13 +1887,14 @@ EOM
 
       class AssertExceptionHelper
         class WrappedException
+          attr_reader :exception
           def initialize(exception)
             @exception = exception
           end
 
           def inspect
             if default_inspect?
-              "#{@exception.class.inspect}(#{@exception.message.inspect})"
+              "#{@exception.class.inspect}(<#{@exception.message}>)"
             else
               @exception.inspect
             end
