@@ -88,6 +88,7 @@ static gboolean real_event_loop_created
 static gchar   *real_dump         (MilterManagerConfiguration *configuration);
 static gboolean real_clear        (MilterManagerConfiguration *configuration,
                                    GError                    **error);
+static GPid     real_fork         (MilterManagerConfiguration *configuration);
 
 static gpointer milter_manager_ruby_configuration_parent_class = NULL;
 static GType    milter_manager_ruby_configuration_type_id = 0;
@@ -117,6 +118,7 @@ milter_manager_ruby_configuration_class_init (MilterManagerRubyConfigurationClas
     configuration_class->event_loop_created = real_event_loop_created;
     configuration_class->dump = real_dump;
     configuration_class->clear = real_clear;
+    configuration_class->fork = real_fork;
 }
 
 static void
@@ -383,6 +385,32 @@ static void
 ruby_cleanup_without_signal_change (int exit_code)
 {
     RETSIGTYPE (*sigint_handler)_((int));
+#ifdef HAVE_RB_THREAD_RESET_TIMER_THREAD
+    const gchar *milter_manager_ruby_reset_timer_thread_before_cleanup = NULL;
+#endif
+#ifdef HAVE_RB_THREAD_STOP_TIMER_THREAD
+    const gchar *milter_manager_ruby_stop_timer_thread_before_cleanup;
+#endif
+
+#ifdef HAVE_RB_THREAD_RESET_TIMER_THREAD
+    milter_manager_ruby_reset_timer_thread_before_cleanup =
+        g_getenv("MILTER_MANAGER_RUBY_RESET_TIMER_THREAD_BEFORE_CLEANUP");
+    if (milter_manager_ruby_reset_timer_thread_before_cleanup &&
+        strcmp(milter_manager_ruby_reset_timer_thread_before_cleanup, "yes") == 0) {
+        void rb_thread_reset_timer_thread(void);
+        rb_thread_reset_timer_thread();
+    }
+#endif
+
+#ifdef HAVE_RB_THREAD_STOP_TIMER_THREAD
+    milter_manager_ruby_stop_timer_thread_before_cleanup =
+        g_getenv("MILTER_MANAGER_RUBY_STOP_TIMER_THREAD_BEFORE_CLEANUP");
+    if (milter_manager_ruby_stop_timer_thread_before_cleanup &&
+        strcmp(milter_manager_ruby_stop_timer_thread_before_cleanup, "yes") == 0) {
+        void rb_thread_stop_timer_thread(void);
+        rb_thread_stop_timer_thread();
+    }
+#endif
 
     sigint_handler = signal(SIGINT, SIG_DFL);
     ruby_cleanup(exit_code);
@@ -583,6 +611,24 @@ real_clear (MilterManagerConfiguration *_configuration, GError **error)
     return success;
 }
 
+static GPid
+real_fork (MilterManagerConfiguration *_configuration)
+{
+#ifdef HAVE_RB_FORK
+    int status;
+    return (GPid)rb_fork(&status, NULL, NULL, Qnil);
+#else
+    VALUE pid;
+
+    pid = rb_funcall2(rb_mKernel, rb_intern("fork"), 0, 0);
+    if (NIL_P(pid)) {
+	return (GPid)0;
+    } else {
+	return (GPid)NUM2INT(pid);
+    }
+#endif
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
